//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  var.cpp
//! \brief Variable reference CVar.


#include <string>

#include "defs.hpp"
#include "var.hpp"


//**************************************************************************************************
//*  class CVar
//**************************************************************************************************

// Variables may be unnumbered (x,y,i,j,k,f,...) or numbered (z0,p0,...).


//! Returns number contained in parameter label s, starting at index pos.

int CVar::request_num( string s, int pos )
{
    if ( s.size()<=pos )
        throw "missing parameter number in '" + s + "'";
    int ret;
    if ( !triv::any2int( s.substr(pos), &ret ) )
        throw "invalid parameter number in '" + s + "'";
    return ret;
}


//! Creates instance according to given parameter label.

CVar::CVar( string s )
{
    num = -1;
    if( s.size()<1 )
        throw S("Variable name missing");

    if ( s[0]=='p' ) {        // curve parameter
        num = request_num( s, 1 );
        typ = _CP;
    } else if ( s=="z+" ) {          // new z value
        typ = _Z;
    } else if ( s[0]=='z' ) {        // existing z value
        num = request_num( s, 1 );
        typ = _Z;
    } else if ( s=="r+" ) {          // new real par
        typ = _R;
    } else if ( s[0]=='r' ) { // existing real par
        num = request_num( s, 1 );
        typ = _R;
    } else if ( s=="nj" ) {
        typ = _NJ;
    } else if ( s=="ni" ) {
        typ = _NI;
    } else if ( s=="x" ) {
        typ = _X;
    } else if ( s=="y" ) {
        typ = _Y;
    } else if ( s=="dy" ) {
        typ = _DY;
    } else if ( s=="c" ) {
        typ = _C;
    } else if ( s=="outcome" ) {
        typ = _COUTCOME;
    } else if ( s=="chi2" ) {
        typ = _CCHI2;
    } else if ( s=="R2" ) {
        typ = _CR2;
    } else if ( s=="k" ) {
        typ = _K;
    } else if ( s=="j" ) {
        typ = _J;
    } else if ( s=="i" ) {
        typ = _I;
    } else {
        throw "Invalid variable '" + s + "'";
        // typ = _NOREF;
    }
}


//! Returns string representation of this variable.

string CVar::var_info() const
{
    string ret;
    switch (typ) {
    case _X:  ret = "x";  break;
    case _Y:  ret = "y";  break;
    case _DY: ret = "dy";  break;
    case _C:  ret = "c";  break;
    case _Z:  ret = "z";  break;
    case _R:  ret = "r";  break;
    case _CP: ret = "p";  break;
    case _COUTCOME: ret = "fit_outcome"; break;
    case _CCHI2: ret = "chi2"; break;
    case _CR2: ret = "R2"; break;
    case _K:  ret = "k";  break;
    case _J:  ret = "j";  break;
    case _I:  ret = "i";  break;
    case _NJ: ret = "nj"; break;
    case _NI: ret = "ni"; break;
    default:
        throw S("BUG: no info available for unexpected variable typ");
    }
    if ( numbered() ) {
        if( num==-1 )
            ret += "+";
        else
            ret += S(num);
    }
    return ret;
}
